@@ -71,7 +71,7 @@ class UserTestCase(TestCase):
             'HTTP_AUTHORIZATION': 'Token {}'.format(self.staff_user_token.key)
         }
 
-        self.valid_new_external_profile = {
+        self.valid_new_external_user = {
             'email': 'cane@gmail.com',
             'username': 'cane',
             'password': 'pippopluto',
@@ -98,6 +98,8 @@ class UserTestCase(TestCase):
             }
         }
 
+        self.valid_profile_update = self.valid_new_external_user['profile']
+
         self.valid_new_profile = {
             'email': 'paperino@gmail.com',
             'username': 'paperino',
@@ -172,10 +174,6 @@ class UserTestCase(TestCase):
             'fingerprint': '123456'
         }
 
-        self.valid_profile_update = {
-            'first_name': 'Pippo'
-        }
-
         self.valid_password_update = {
             'password': updated_password
         }
