@@ -4,28 +4,26 @@ import os
 from django.utils.decorators import method_decorator
 from django.views.decorators.debug import sensitive_post_parameters
 from django.views.decorators.csrf import csrf_exempt
-from django.contrib.auth import logout as django_logout
-# from django.http import HttpResponseRedirect
+from django.shortcuts import reverse
 
 from rest_framework import status, serializers
-from rest_framework.generics import GenericAPIView
+from rest_framework.views import APIView
 from rest_framework.permissions import IsAuthenticated, AllowAny
 from rest_framework.response import Response
 
-from allauth.account import views as allauth_account_views
 from allauth.account.models import EmailAddress
 from allauth.account.views import INTERNAL_RESET_URL_KEY, INTERNAL_RESET_SESSION_KEY
 from allauth.account.forms import UserTokenForm
 from allauth.utils import get_request_param
+from allauth.account.adapter import get_adapter
 
 from ..apps.users.serializers import SuccessfullLoginResponseSerializer, SuccessfullLogoutResponseSerializer
 from ..apps.passepartout.utils import get_passepartout_login_redirect_url
-from ..permissions import is_staff, is_authenticated
+from ..permissions import is_authenticated
+from ..utils import set_session_key, get_session_key, get_random_fingerprint
+from .. import views as django_sso_app_views
 from .. import app_settings
 from .serializers import LoginSerializer, SignupSerializer, PasswordResetSerializer, PasswordResetFromKeySerializer
-from ..utils import invalidate_cookie, set_cookie, set_session_key, get_session_key, is_test_user
-from .utils import get_request_messages_string, get_error_response_data
-
 
 sensitive_post_parameters_m = method_decorator(
     sensitive_post_parameters('password', 'password1', 'password2'))
@@ -34,32 +32,64 @@ logger = logging.getLogger('django_sso_app.core.api')
 
 CURRENT_DIR = os.getcwd()
 SUCCESS_STATUS_CODES = (status.HTTP_200_OK, status.HTTP_201_CREATED, status.HTTP_302_FOUND)
+EMAIL_NOT_VERIFIED_MESSAGE = get_adapter().error_messages['email_not_verified']
 
 
-class FormWrapperViewMixin(GenericAPIView):
+class DjangoSsoAppApiViewMixin(APIView):
+    # Form wrapper mixin
+
     http_method_names = ['get', 'post', 'head', 'options']
-
     _initial = {}
+    response_errors = ['Server error.']
+    response_error_status = status.HTTP_500_INTERNAL_SERVER_ERROR
+    response_redirects_to_email_confirmation = False
 
     def get_initial(self):
         """Return the initial data to use for forms on this view."""
         return self._initial.copy()
 
+    def get_request_fingerprint(self, request):
+        return request.data.get('fingerprint', get_random_fingerprint(request))
+
+    def get_error_response(self):
+        return Response(self.response_errors, status=self.response_error_status)
+
     @method_decorator(csrf_exempt)
     def dispatch(self, request, *args, **kwargs):
-        return super(FormWrapperViewMixin, self).dispatch(request, *args, **kwargs)
+        return super(DjangoSsoAppApiViewMixin, self).dispatch(request, *args, **kwargs)
+
+    def form_valid(self, form):
+        response = super(DjangoSsoAppApiViewMixin, self).form_valid(form)
+        status_code = response.status_code
+
+        if status_code not in SUCCESS_STATUS_CODES:
+            raise Exception('Wrong status code "{}"'.format(status_code))
+
+        location = response.get('location', None)
+        if location == reverse('account_email_verification_sent'):
+            self.response_redirects_to_email_confirmation = True
+
+        return response
+
+    def form_invalid(self, form):
+        form_errors = dict(form.errors.items())
+
+        self.response_error_status = status.HTTP_400_BAD_REQUEST
+        self.response_errors = form_errors
+
+        return Response(self.response_errors, status=self.response_error_status)
 
 
 # allauth views
 
-class LoginView(FormWrapperViewMixin, allauth_account_views.LoginView):
+class LoginView(DjangoSsoAppApiViewMixin, django_sso_app_views.LoginView):
     """
     Login
     """
     permission_classes = (AllowAny, )
     serializer_class = LoginSerializer
     http_method_names = ['post', 'head', 'options']
-    response_data_errors = ['Login error.']
+    email_not_verified_errors = [EMAIL_NOT_VERIFIED_MESSAGE]
 
     def get_form_kwargs(self, **kwargs):
         kwargs = super(LoginView, self).get_form_kwargs(**kwargs)
@@ -67,12 +97,12 @@ class LoginView(FormWrapperViewMixin, allauth_account_views.LoginView):
         kwargs['data'] ={
             'login': self.request.data.get('login', None),
             'password': self.request.data.get('password', None),
-            'fingerprint': self.request.data.get('fingerprint', None),
+            'fingerprint': self.request.data.get('fingerprint', get_random_fingerprint(self.request)),
         }
 
         return kwargs
 
-    def get_response(self):
+    def get_success_response(self):
         request = self.request
         data = {
             'user': request.user,
@@ -86,71 +116,60 @@ class LoginView(FormWrapperViewMixin, allauth_account_views.LoginView):
 
         return response
 
-    def form_valid(self, form):
-        logger.debug('Valid login API form')
-        return super(LoginView, self).form_valid(form)
-
-    def form_invalid(self, form):
-        errors = getattr(form, '__django_sso_app__login_errors', self.response_data_errors)
-        logger.info('Invalid login API form "{}"'.format(errors))
-
-        self.response_data_errors = errors
-
-        return super(LoginView, self).form_invalid(form)
-
     def post(self, request, *args, **kwargs):
         logger.info('API logging in')
+
         if is_authenticated(request.user):
             return Response(status=status.HTTP_400_BAD_REQUEST)
 
         try:
-            set_session_key(request, '__django_sso_app__device__fingerprint', request.data.get('fingerprint', None))
+            set_session_key(request, '__django_sso_app__device__fingerprint', self.get_request_fingerprint(request))
 
             _response = super(LoginView, self).post(request, *args, **kwargs)
             _status_code = _response.status_code
 
-            logger.debug('allauth response ({})'.format(_response.__dict__))
+            # check user must confirm email
+            if self.response_redirects_to_email_confirmation:
+                self.response_error_status = status.HTTP_400_BAD_REQUEST
+                self.response_errors = self.email_not_verified_errors
 
-            if _status_code not in SUCCESS_STATUS_CODES:
-                logger.debug('allauth failed response')
+                raise Exception('Email not verified')
 
         except:
             logger.exception('API Error logging in')
-            return Response(status=status.HTTP_500_INTERNAL_SERVER_ERROR)
+            return self.get_error_response()
 
         else:
             if is_authenticated(request.user):
-                logger.debug('valid login')
-
-                response = self.get_response()
-
-                if not is_staff(request.user):
-                    token = get_session_key(request, '__django_sso_app__jwt_token')
-                    set_cookie(response, app_settings.JWT_COOKIE_NAME, token, None)
-
+                response = self.get_success_response()
             else:
-                logger.info('invalid login')
+                logger.info('user "{}" is not logged in'.format(request.user))
 
-                try:
-                    response_data = get_error_response_data(_response)
+                # check user has unsubscribed
+                user_unsubscribed_at = get_session_key(request, '__django_sso_app__user_is_unsubscribed', None)
+                if user_unsubscribed_at is not None:
+                    self.response_error_status = status.HTTP_400_BAD_REQUEST
+                    self.response_errors = ['Unsubscribed at "{}"'.format(user_unsubscribed_at)]
 
-                except:
-                    logger.exception('login error')
-                    response_data = ', '.join(self.response_data_errors)
+                # check user must confirm email
+                if _response.get('location', None) == reverse('account_email_verification_sent'):
+                    self.response_error_status = status.HTTP_400_BAD_REQUEST
+                    self.response_errors = self.email_not_verified_errors
 
-                response = Response(response_data, status.HTTP_400_BAD_REQUEST)
+                return self.get_error_response()
 
-            return response
+            return self.get_response_with_cookie(request, response)
 
 
-class LogoutView(FormWrapperViewMixin, allauth_account_views.LogoutView):
+class LogoutView(DjangoSsoAppApiViewMixin, django_sso_app_views.LogoutView):
     """
     Logout
     """
     permission_classes = (IsAuthenticated, )
     serializer_class = serializers.Serializer
+    http_method_names = ['post', 'head', 'options']
 
-    def get_response(self):
+    def get_success_response(self):
         request = self.request
         redirect_url = get_request_param(request, 'next')
         data = {
@@ -159,59 +178,43 @@ class LogoutView(FormWrapperViewMixin, allauth_account_views.LogoutView):
         serializer = SuccessfullLogoutResponseSerializer(instance=data,
                                                          context={'request': self.request})
 
-        # response = HttpResponseRedirect(redirect_url)
-        response = Response(serializer.data, status=status.HTTP_200_OK)
-
-        return response
-
-    def get(self, request, *args, **kwargs):
-        logger.info('API Getting logout')
-        if request.user and request.user.is_anonymous:
-            return Response(status=status.HTTP_400_BAD_REQUEST)
-
-        try:
-            return super(LogoutView, self).get(request, *args, **kwargs)
-
-        except:
-            logger.exception('API Error getting logout')
-            raise
+        return Response(serializer.data, status=status.HTTP_200_OK)
 
     def post(self, request, *args, **kwargs):
         logger.info('API Logging out')
+
         if request.user and request.user.is_anonymous:
             return Response(status=status.HTTP_400_BAD_REQUEST)
 
         try:
             _response = super(LogoutView, self).post(request, *args, **kwargs)
-            status_code = _response.status_code
-            if status_code not in SUCCESS_STATUS_CODES:
-                logger.info('status_code error "{}"'.format(status_code))
+            _status_code = _response.status_code
+
+            if _status_code not in SUCCESS_STATUS_CODES:
+                logger.info('LogoutView status_code error ({})'.format(_status_code))
 
         except:
             logger.exception('API Error logging out')
-            return Response(status=status.HTTP_500_INTERNAL_SERVER_ERROR)
+            return self.get_error_response()
 
         else:
-            response = self.get_response()
-
-            invalidate_cookie(response, app_settings.JWT_COOKIE_NAME)
-
-            return response
+            return self.get_response_with_invalidated_cookie(self.get_success_response())
 
 
-class SignupView(FormWrapperViewMixin, allauth_account_views.SignupView):
+class SignupView(DjangoSsoAppApiViewMixin, django_sso_app_views.SignupView):
     """
     Signup
     """
     permission_classes = (AllowAny, )
     serializer_class = SignupSerializer
     http_method_names = ['head', 'post', 'options']
+    signup_form_fields = app_settings.REQUIRED_USER_FIELDS + ('password1', 'password2', 'fingerprint', 'referrer')
 
     def get_form_kwargs(self, **kwargs):
         kwargs = super(SignupView, self).get_form_kwargs(**kwargs)
 
         data = {}
-        for field in app_settings.REQUIRED_USER_FIELDS + ('password1', 'password2', 'fingerprint', 'referrer'):
+        for field in self.signup_form_fields:
             data[field] = self.request.data.get(field, None)
 
         if app_settings.BACKEND_SIGNUP_MUST_FILL_PROFILE:
@@ -222,43 +225,24 @@ class SignupView(FormWrapperViewMixin, allauth_account_views.SignupView):
 
         return kwargs
 
-    def form_valid(self, form):
-        # By assigning the User to a property on the view, we allow subclasses
-        # of SignupView to access the newly created User instance
-
-        logger.debug('Valid signup API form')
-        return super(SignupView, self).form_valid(form)
-
     def post(self, request, *args, **kwargs):
         logger.info('API Signing up')
+
         if is_authenticated(request.user):
             return Response(status=status.HTTP_400_BAD_REQUEST)
 
         try:
-            _response = super(SignupView, self).post(request, *args, **kwargs)
-            _status_code = _response.status_code
-            logger.debug('allauth response ({})'.format(_status_code))
+            response = super(SignupView, self).post(request, *args, **kwargs)
 
         except:
             logger.exception('API Error signin up')
-            return Response(status=status.HTTP_500_INTERNAL_SERVER_ERROR)
+            return self.get_error_response()
 
         else:
-            if _status_code not in SUCCESS_STATUS_CODES:
-                response_data = get_error_response_data(_response)
-
-                response = Response(response_data, status.HTTP_400_BAD_REQUEST)
-            else:
-                response = Response(status=status.HTTP_201_CREATED)
-
-            if is_authenticated(request.user):
-                logger.debug('disable API SignupView login')
-                django_logout(request)
-
             return response
 
 
-class EmailView(FormWrapperViewMixin, allauth_account_views.EmailView):
+class EmailView(DjangoSsoAppApiViewMixin, django_sso_app_views.EmailView):
     """
     Manage user email addresses
     """
@@ -268,12 +252,13 @@ class EmailView(FormWrapperViewMixin, allauth_account_views.EmailView):
 
     def get(self, request, *args, **kwargs):
         logger.info('API Getting email')
+
         try:
             _response = super(EmailView, self).get(request, *args, **kwargs)
 
         except:
             logger.exception('API Error getting email')
-            raise
+            return self.get_error_response()
 
         else:
             email_addresses = EmailAddress.objects.filter(user=request.user).count()
@@ -281,7 +266,7 @@ class EmailView(FormWrapperViewMixin, allauth_account_views.EmailView):
             return Response(email_addresses, status=status.HTTP_200_OK)
 
 
-class PasswordResetView(FormWrapperViewMixin, allauth_account_views.PasswordResetView):
+class PasswordResetView(DjangoSsoAppApiViewMixin, django_sso_app_views.PasswordResetView):
     """
     Reset user password
     """
@@ -291,6 +276,7 @@ class PasswordResetView(FormWrapperViewMixin, allauth_account_views.PasswordRese
 
     def get_form_kwargs(self, **kwargs):
         kwargs = super(PasswordResetView, self).get_form_kwargs(**kwargs)
+
         kwargs['data'] = {
             'email': self.request.data.get('email', None)
         }
@@ -301,14 +287,16 @@ class PasswordResetView(FormWrapperViewMixin, allauth_account_views.PasswordRese
         logger.info('API Asking for password reset')
         try:
             _response = super(PasswordResetView, self).post(request, *args, **kwargs)
+
         except:
             logger.exception('API Error while asking password reset')
-            raise
+            return self.get_error_response()
+
         else:
             return _response
 
 
-class PasswordResetFromKeyView(FormWrapperViewMixin, allauth_account_views.PasswordResetFromKeyView):
+class PasswordResetFromKeyView(DjangoSsoAppApiViewMixin, django_sso_app_views.PasswordResetFromKeyView):
     """
     Confirm user password reset
     """
@@ -324,8 +312,10 @@ class PasswordResetFromKeyView(FormWrapperViewMixin, allauth_account_views.Passw
 
         token_form = UserTokenForm(
             data={'uidb36': uidb36, 'key': self.key})
+
         if token_form.is_valid():
             self.reset_user = token_form.reset_user
+
         else:
             logger.error('key is not valid!')
             raise KeyError('key is not valid')
@@ -344,9 +334,11 @@ class PasswordResetFromKeyView(FormWrapperViewMixin, allauth_account_views.Passw
         try:
             set_session_key(request, INTERNAL_RESET_SESSION_KEY, key)
 
-            _response = super(PasswordResetFromKeyView, self).post(request, uidb36, INTERNAL_RESET_URL_KEY, **kwargs)
+            response = super(PasswordResetFromKeyView, self).post(request, uidb36, INTERNAL_RESET_URL_KEY, **kwargs)
+
         except:
             logger.exception('Error while resetting password form key')
-            raise
+            return self.get_error_response()
+
         else:
-            return _response
+            return response
