@@ -266,33 +266,56 @@ class NewUserSerializer(UserSerializer):
 
         assert (new_password is not None)
 
-        logger.info('User "{}" wants to create new user "{}"'.format(requesting_user, validated_data))
-
-        if requesting_user_is_staff and skip_confirmation is not None:
-            must_confirm_email = False
-
-        new_user = create_local_user_from_object(validated_data, not must_confirm_email)
-
-        # setting new_user profile serializer field
-        setattr(new_user, 'profile', update_profile(new_user.sso_app_profile, validated_data.get('profile')))
-
-        if password_is_hashed:
-            new_user.password = new_password
-            new_user.save()
-
+        # start creating new user
+        new_user = None
         try:
-            new_user_subscriptions =  request.data.get('profile', {}).get('subscriptions', [])
-            if len(new_user_subscriptions) > 0:
-                for new_subscription in new_user_subscriptions:
-                    new_subscription_service_name = new_subscription.get('service').get('name')
-                    service = Service.objects.get(name=new_subscription_service_name)
-                    adapter.subscribe_profile_to_service(new_user.sso_app_profile, service)
+            logger.info('User "{}" wants to create new user "{}"'.format(requesting_user, validated_data))
 
-        except Exception:
-            logger.exception('Error subscribing service on user creation')
-            raise serializers.ValidationError('Error subscribing service')
+            if requesting_user_is_staff and skip_confirmation is not None:
+                must_confirm_email = False
 
-        return new_user
+            new_user = create_local_user_from_object(validated_data, verified=(not must_confirm_email))
+            new_user_profile_validated_object = validated_data.get('profile', {})
+
+            # setting new_user profile serializer field
+            setattr(new_user, 'profile', update_profile(new_user.sso_app_profile, new_user_profile_validated_object))
+
+            if password_is_hashed:
+                logger.info('New user password is hashed')
+                new_user.password = new_password
+                new_user.save()
+
+            try:
+                new_user_profile_request_object = request.data.get('profile', {})
+                profile_object_subscriptions = new_user_profile_request_object.get('subscriptions', [])
+
+                logger.info('Subscripting new user to services "{}"'.format(profile_object_subscriptions))
+
+                if len(profile_object_subscriptions) > 0:
+                    for new_subscription in profile_object_subscriptions:
+                        new_subscription_service_name = new_subscription.get('service').get('name')
+                        service = Service.objects.get(name=new_subscription_service_name)
+                        adapter.subscribe_profile_to_service(new_user.sso_app_profile, service)
+
+            except Exception:
+                logger.exception('Error subscribing service on user creation')
+                raise serializers.ValidationError('Error subscribing service')
+
+            if must_confirm_email:
+                logger.info('Sending confirmation email to new user "{}"'.format(new_user))
+                new_user_email_address = new_user.emailaddress_set.last()
+                new_user_email_address.send_confirmation(request=request, signup=True)
+
+            return new_user
+
+        # delete creating user model if any exception occurred
+        except Exception as e:
+            logger.exception('Error creating new user "{}"'.format(e))
+
+            if new_user is not None:
+                new_user.delete()
+
+            raise
 
 
 class SuccessfullLoginResponseSerializer(serializers.Serializer):
