@@ -11,6 +11,8 @@ from rest_framework import status
 
 from django_sso_app.backend.tests.factories import UserTestCase
 
+from .... import app_settings
+
 User = get_user_model()
 
 
@@ -20,7 +22,7 @@ class TestUserCreate(UserTestCase):
         print('HEADERS', self.staff_user_valid_token_headers, type(self.staff_user_valid_token_headers))
         response = self.client.post(
             reverse('django_sso_app_user:list'),
-            data=json.dumps(self.valid_new_external_profile),
+            data=json.dumps(self.valid_new_external_user),
             **self.staff_user_valid_token_headers
         )
 
@@ -35,7 +37,7 @@ class TestUserCreate(UserTestCase):
 
         self.assertEqual(response2.status_code, status.HTTP_200_OK)
 
-        self.assertEqual(response2.data['email'], self.valid_new_external_profile['email'])
+        self.assertEqual(response2.data['email'], self.valid_new_external_user['email'])
 
         self.assertEqual(response2.data['email_verified'], False)
 
@@ -44,7 +46,7 @@ class TestUserCreate(UserTestCase):
 
         response = self.client.post(
             reverse('django_sso_app_user:list') + '?skip_confirmation=true',
-            data=json.dumps(self.valid_new_external_profile),
+            data=json.dumps(self.valid_new_external_user),
             **self.staff_user_valid_token_headers
         )
 
@@ -60,7 +62,7 @@ class TestUserCreate(UserTestCase):
         self.assertEqual(response2.status_code, status.HTTP_200_OK)
 
 
-        self.assertEqual(response2.data['email'], self.valid_new_external_profile['email'])
+        self.assertEqual(response2.data['email'], self.valid_new_external_user['email'])
 
         self.assertEqual(response2.data['email_verified'], True)
 
@@ -73,7 +75,7 @@ class TestUserCreate(UserTestCase):
 
         response = self.client.post(
             reverse('django_sso_app_user:list') + '?skip_confirmation=true',
-            data=json.dumps(self.valid_new_external_profile),
+            data=json.dumps(self.valid_new_external_user),
             content_type='application/json'
         )
 
@@ -90,7 +92,7 @@ class TestUserCreate(UserTestCase):
 
         self.assertEqual(response2.status_code, status.HTTP_200_OK)
 
-        self.assertEqual(response2.data['email'], self.valid_new_external_profile['email'])
+        self.assertEqual(response2.data['email'], self.valid_new_external_user['email'])
 
         self.assertEqual(response2.data['email_verified'], False)
 
@@ -100,6 +102,18 @@ class TestUserCreate(UserTestCase):
         self.assertEqual(new_user.sso_app_profile.subscriptions.first().service, self.service2 ,
                          'no subscription created')
 
+        profile_object = self.valid_new_external_user['profile']
+        for f in app_settings.PROFILE_FIELDS:
+            model_val = getattr(new_user.sso_app_profile, f, None)
+            object_val = profile_object.get(f, None)
+
+            if f == 'birthdate':
+                model_val = str(model_val)
+            elif f == 'country':
+                object_val = object_val.upper()
+                model_val = str(model_val).upper()
+
+            self.assertEqual(model_val, object_val)
 
 
 class TestUserRetrieve(UserTestCase):
