@@ -7,10 +7,12 @@ from django.db import transaction
 from django.utils.encoding import smart_str
 from django.contrib.auth import get_user_model
 
+from allauth.account.models import EmailAddress
+
 from ... import app_settings
 from ..api_gateway.kong.functions import get_apigateway_sso_id
-from ..profiles.models import Profile
 from ..profiles.utils import update_profile, get_or_create_user_profile
+from ..profiles.models import Profile
 
 logger = logging.getLogger('django_sso_app.core.apps.users')
 User = get_user_model()
@@ -111,7 +113,7 @@ def check_remote_user_existence_by_email(email):
 
 
 def get_remote_user_by_sso_id(sso_id):
-    logger.info('getting remote user with sso_id "{}"'.format(sso_id))
+    logger.info('Getting remote user with sso_id "{}"'.format(sso_id))
 
     url = app_settings.REMOTE_USER_URL.format(sso_id=sso_id) + '?with_password=true'
     params = {
@@ -135,69 +137,124 @@ def get_remote_user_by_sso_id(sso_id):
     return response_data
 
 
+def create_local_user_from_object(user_object, verified=False):
+    logger.info('Creating local user from object "{}". Verified: {}'.format(user_object, verified))
+
+    username = user_object.get('username')
+    email = user_object.get('email')
+    sso_id = user_object.get('sso_id', None)
+    sso_rev = user_object.get('sso_rev', 0)
+
+    if sso_id is not None:
+        sso_id = smart_str(sso_id)
+
+        if User.objects.filter(sso_app_profile__sso_id=sso_id).count() > 0:
+            raise Exception('Profile with sso_id "{}" already present'.format(sso_id))
+
+        logger.info('Creating local user with sso_id "{}" and sso_rev "{}" ...'.format(sso_id, sso_rev))
+
+    if User.objects.filter(username=username).count() > 0:
+        raise Exception('User with username "{}" already present'.format(username))
+
+    if User.objects.filter(email=email).count() > 0:
+        raise Exception('User with email "{}" already present'.format(email))
+
+    new_user = User()
+
+    setattr(new_user, '__django_sso_app__creating', True)
+    setattr(new_user, '__django_sso_app__profile__sso_id', sso_id)
+    setattr(new_user, '__django_sso_app__profile__sso_rev', sso_rev)
+
+    # saving model there (commit) implies profile creation with pre-defined attrs sso_id, sso_rev
+    new_user = update_user(new_user, user_object, commit=True)
+
+    email = user_object.get('email', None)
+
+    if email is not None:
+        if EmailAddress.objects.filter(email__iexact=email).count() > 0:
+            raise Exception('EmailAddress already present')
+
+        created_email = EmailAddress.objects.create(user=new_user,
+                                                    email=email,
+                                                    verified=verified,
+                                                    primary=True)
+        logger.info(
+            'A new EmailAddress for {0} has been created with id {1}'
+            ''.format(
+                email, created_email.id))
+
+        if verified:
+            logger.info(
+                '{0} must NOT confirm email {1}'.format(new_user, email))
+        else:
+            logger.info(
+                '{0} must confirm email {1}'.format(new_user, email))
+
+    logger.info('local user created "{}"'.format(new_user))
+
+    return new_user
+
+
 @transaction.atomic
 def create_local_user_from_remote_backend(remote_user, can_subscribe=False):
-    remote_object_profile = remote_user['profile']
+    # Creates new user only by profile object
+    remote_user_profile_object = remote_user.get('profile')
 
-    sso_id = smart_str(remote_object_profile['sso_id'])
-    sso_rev = remote_object_profile['sso_rev']
+    new_user = create_local_user_from_object(remote_user_profile_object,
+                                             verified=getattr(remote_user, 'email_verified', False))
 
-    logger.info('Creating local user with sso_id "{}" and sso_rev "{}" ...'.format(sso_id, sso_rev))
+    setattr(new_user, '__django_sso_app__remote_user', remote_user)  # noqa (subscription required)
 
-    user = User()
-    user = update_user(user, remote_user, commit=False)
+    setattr(new_user, 'sso_app_profile', get_or_create_user_profile(new_user, Profile, remote_user_profile_object))
 
-    setattr(user, '__django_sso_app__profile__sso_id', sso_id)
-    setattr(user, '__django_sso_app__profile__sso_rev', sso_rev)
-    setattr(user, '__django_sso_app__remote_user', remote_user)
-
-    setattr(user, '__django_sso_app__creating', True)
-
-    user.save() # saving model there implies profile creation with pre-defined attrs sso_id, sso_rev
-    logger.info('local user created "{}"'.format(user))
-
-    return user
+    return new_user
 
 
 @transaction.atomic
 def create_local_user_from_apigateway_headers(request):
     consumer_username = request.META.get(app_settings.APIGATEWAY_CONSUMER_USERNAME_HEADER)
-
     logger.info('Creating local user from apigateway consumer username "{}"'.format(smart_str(consumer_username)))
 
     sso_id = get_apigateway_sso_id(consumer_username)
-    temporary_email = '{}@{}'.format(sso_id, app_settings.COOKIE_DOMAIN)
+    sso_rev = 0
+    username = consumer_username
+    temporary_email = '{}@{}'.format(sso_id, app_settings.APP_DOMAIN)
 
-    user = User(username=sso_id, email=temporary_email)
+    profile_object = {
+        'sso_id': sso_id,
+        'sso_rev': sso_rev,
+        'username': username,
+        'email': temporary_email
+    }
 
-    setattr(user, '__django_sso_app__creating', True)
-    setattr(user, '__django_sso_app__profile__sso_id', sso_id)
-    setattr(user, '__django_sso_app__profile__sso_rev', 0)
+    new_user = create_local_user_from_object(profile_object)
 
-    user.save()
-    logger.info('local user created "{}"'.format(user))
+    #setattr(new_user, 'sso_app_profile', get_or_create_user_profile(new_user, Profile, profile_object))
 
-    return user
+    return new_user
 
 
 @transaction.atomic
 def create_local_user_from_jwt(decoded_jwt):
-    sso_id = decoded_jwt['sso_id']
-    sso_rev = decoded_jwt['sso_rev']
+    logger.info('Creating local user from jwt')
+
+    sso_id = decoded_jwt.get('sso_id')
+    sso_rev = decoded_jwt.get('sso_rev')
     username = decoded_jwt.get('username', sso_id)
-    # email = decoded_jwt.get('email', None)
+    temporary_email = '{}@{}'.format(sso_id, app_settings.APP_DOMAIN)
 
-    temporary_email = '{}@{}'.format(sso_id, app_settings.COOKIE_DOMAIN)
-    user = User(username=username, email=temporary_email)
+    profile_object = {
+        'sso_id': sso_id,
+        'sso_rev': sso_rev,
+        'username': username,
+        'email': temporary_email
+    }
 
-    setattr(user, '__django_sso_app__creating', True)
-    setattr(user, '__django_sso_app__profile__sso_id', sso_id)
-    setattr(user, '__django_sso_app__profile__sso_rev', sso_rev)
+    new_user = create_local_user_from_object(profile_object)
 
-    user.save()
-    logger.info('local user created "{}"'.format(user))
+    #setattr(new_user, 'sso_app_profile', get_or_create_user_profile(new_user, Profile, profile_object))
 
-    return user
+    return new_user
 
 
 @transaction.atomic
@@ -212,7 +269,6 @@ def update_local_user_from_remote_backend(remote_user, profile, commit=True):
 
     setattr(user, '__django_sso_app__remote_user', remote_user)
 
-    _profile = update_profile(profile, remote_user['profile'], commit)
-    logger.info('Local profile "{}" updated'.format(_profile))
+    setattr(user, 'sso_app_profile', update_profile(profile, remote_object_profile, commit))
 
     return user
