@@ -9,6 +9,9 @@ from django.utils.decorators import method_decorator
 from django.utils.translation import ugettext_lazy as _
 from django.views.decorators.cache import cache_page
 from django.db.models import Q
+from django.core.exceptions import SuspiciousOperation
+
+from allauth.account.adapter import get_adapter
 
 from rest_framework import generics
 from rest_framework import mixins
@@ -21,12 +24,11 @@ from rest_framework.views import APIView
 from allauth.account.models import EmailAddress
 
 from .serializers import (CheckUserExistenceSerializer,
-                          UserSerializer,
+                          UserSerializer, NewUserSerializer,
                           UserRevisionSerializer)
 from ...permissions import StaffPermission
 from ...permissions import is_staff
-from ...tokens.utils import get_request_jwt, renew_response_jwt
-from ...utils import is_test_user, invalidate_cookie
+from ...utils import is_test_user, invalidate_cookie, get_session_key
 from ... import app_settings
 
 User = get_user_model()
@@ -43,15 +45,23 @@ class CheckUserExistenceApiView(APIView):
     def get_object(self, login=None, username=None, email=None):
         try:
             if login is not None:
-                users = User.objects.filter(Q(username=login) | Q(email=login))
-                return users.first()
+                user = User.objects.filter(Q(username=login) | Q(email=login)).first()
+                if user is not None:
+                    return user
+
+                email_address = EmailAddress.objects.filter(email=login, verified=True).first()
+                if email_address is not None:
+                    return email_address.user
+
             elif username is None and email is not None:
-                email_address = EmailAddress.objects.get(email=email)
-                return email_address.user
+                email_address = EmailAddress.objects.filter(email=email, verified=True).first()
+                if email_address is not None:
+                    return email_address.user
+
             elif username is not None and email is None:
                 return User.objects.get(username=username)
 
-            raise User.DoesNotExist('Either username or email must be set.')
+            raise User.DoesNotExist('Either login username or email must be set.')
 
         except (User.DoesNotExist, EmailAddress.DoesNotExist):
             return None
@@ -85,7 +95,13 @@ class UserApiViewSet(mixins.ListModelMixin,
     """
 
     lookup_field = 'sso_app_profile__sso_id'
-    serializer_class = UserSerializer
+    # serializer_class = UserSerializer
+
+    def get_serializer_class(self):
+        if self.request.method == 'POST':
+            return NewUserSerializer
+        else:
+            return UserSerializer
 
     def get_queryset(self):
         user = self.request.user
@@ -125,7 +141,6 @@ class UserApiViewSet(mixins.ListModelMixin,
 
     def partial_update(self, request, sso_id, *args, **kwargs):
         logger.info('Updating user')
-        requesting_user = self.request.user
 
         try:
             response = super(UserApiViewSet, self).partial_update(request, sso_id, *args, **kwargs)
@@ -141,15 +156,24 @@ class UserApiViewSet(mixins.ListModelMixin,
         logger.info('Partial_update status code is {}, {}'.format(response_status_code, request.data))
 
         if response_status_code == 200:
+            adapter = get_adapter(request)
             user = self.get_object()
+            requesting_user = request.user
 
-            received_jwt = get_request_jwt(request)
-            from_browser_as_same_user = received_jwt is not None and requesting_user == user
+            if get_session_key(request, '__django_sso_app__user_email_updated', False):
+                is_same_user = requesting_user == user
 
-            if from_browser_as_same_user:  # From browser as same user
-                logger.info('From browser as same user "{}", updating JWT'.format(user))
+                if is_same_user:
+                    # loggin out after email update
+                    adapter.logout(request)
+                else:
+                    if not is_staff(requesting_user):
+                        _msg = 'User "{}" tried to updated email for "{}"'.format(requesting_user, user)
+                        logger.warning(_msg)
 
-                renew_response_jwt(received_jwt, user, request, response)
+                        raise SuspiciousOperation(_msg)
+            else:
+                adapter.try_update_response_jwt(user, response)
 
         return response
 
@@ -185,7 +209,7 @@ class UserRevisionsApiView(generics.ListAPIView):
     User revisions entrypoint
     """
 
-    queryset = User.objects.all()
+    queryset = User.objects.filter(is_superuser=False)
     serializer_class = UserRevisionSerializer
     permission_classes = (StaffPermission,)
 
