@@ -32,12 +32,6 @@ class WebpackBuiltTemplateViewMixin(TemplateView):
     Uses built template view when DJANGO_SSO_APP_BACKEND_HAS_CUSTOM_FRONTEND_APP
     """
 
-    def __init__(self, *args, **kwargs):
-        super(WebpackBuiltTemplateViewMixin, self).__init__(*args, **kwargs)
-
-        if not hasattr(self, 'object'):
-            setattr(self, 'object', None)
-
     def get_template_names(self):
         if app_settings.BACKEND_HAS_CUSTOM_FRONTEND_APP:
             return ['frontend/built.html']
